@@ -1,3 +1,4 @@
+from collections import defaultdict
 from functools import reduce, partial
 import operator
 from typing import Any, Dict, Union, Tuple, cast, Optional, Iterator
@@ -11,6 +12,8 @@ import fastremap
 import numpy as np
 from tqdm import tqdm
 
+from mapbuffer import MapBuffer
+
 from cloudfiles import CloudFiles
 
 import cloudvolume
@@ -25,7 +28,8 @@ from igneous.tasks import (
   TransferTask, TouchTask, LuminanceLevelsTask,
   HyperSquareConsensusTask, # HyperSquareTask,
   ImageShardTransferTask, ImageShardDownsampleTask,
-  CCLFacesTask, CCLEquivalancesTask, RelabelCCLTask
+  CCLFacesTask, CCLEquivalancesTask, RelabelCCLTask,
+  CountVoxelsTask
 )
 
 from igneous.shards import image_shard_shape_from_spec
@@ -53,6 +57,8 @@ __all__  = [
   "create_ccl_face_tasks",
   "create_ccl_equivalence_tasks",
   "create_ccl_relabel_tasks",
+  "create_voxel_counting_tasks",
+  "accumulate_voxel_counts",
 ]
 
 # A reasonable size for processing large
@@ -272,7 +278,8 @@ def create_sharded_image_info(
   uncompressed_shard_bytesize: int = MEMORY_TARGET, 
   max_shard_index_bytes: int = 8192, # 2^13
   max_minishard_index_bytes: int = 40000,
-  max_labels_per_minishard: int = 4000
+  max_labels_per_minishard: int = 4000,
+  data_encoding:str = "gzip"
 ) -> Dict[str, Any]:
   """
   Create a recommended sharding scheme. These recommendations are based
@@ -395,8 +402,7 @@ def create_sharded_image_info(
   if preshift_bits + shard_bits + minishard_bits > max_bits:
     raise ValueError(f"{preshift_bits} preshift_bits {shard_bits} shard_bits + {minishard_bits} minishard_bits must be <= {max_bits}. Try reducing the number of minishards.")
 
-  data_encoding = "gzip"
-  if encoding in ("jpeg", "kempressed", "fpzip"):
+  if encoding in ("jpeg", "kempressed", "fpzip", "zfpc"):
     data_encoding = "raw"
 
   return {
@@ -426,22 +432,29 @@ def create_image_shard_transfer_tasks(
   clean_info: bool = False,
   encoding_level: Optional[int] = None,
   truncate_scales: bool = True,
+  compress:bool = True,
+  cutout:bool = False,
 ):
   src_vol = CloudVolume(src_layer_path, mip=mip)
 
   if dest_voxel_offset:
     dest_voxel_offset = Vec(*dest_voxel_offset, dtype=int)
+  elif cutout and bounds:
+    dest_voxel_offset = list(bounds.minpt)
   else:
     dest_voxel_offset = src_vol.voxel_offset.clone()
 
   if not chunk_size:
     chunk_size = src_vol.info['scales'][mip]['chunk_sizes'][0]
   chunk_size = Vec(*chunk_size)
-
+  
   try:
     dest_vol = CloudVolume(dst_layer_path, mip=mip)
   except cloudvolume.exceptions.InfoUnavailableError:
     info = copy.deepcopy(src_vol.info)
+    if cutout and bounds:
+      info['scales'][mip]["voxel_offset"] = list(bounds.minpt)
+      info['scales'][mip]["size"] = list(bounds.size3())
     dest_vol = CloudVolume(dst_layer_path, info=info, mip=mip)
     dest_vol.commit_info()
   except cloudvolume.exceptions.ScaleUnavailableError:
@@ -472,6 +485,7 @@ def create_image_shard_transfer_tasks(
     encoding=dest_vol.scale["encoding"], 
     dtype=dest_vol.dtype,
     uncompressed_shard_bytesize=memory_target,
+    data_encoding=("gzip" if compress else "raw"),
   )
   dest_vol.scale["sharding"] = spec
   if clean_info:
@@ -676,6 +690,7 @@ def create_transfer_tasks(
   bounds_mip:int = 0,
   encoding_level:Optional[int] = None,
   truncate_scales:bool = True,
+  cutout:bool = False,
 ) -> Iterator:
   """
   Transfer data to a new data layer. You can use this operation
@@ -689,6 +704,8 @@ def create_transfer_tasks(
 
   bounds: Bbox specified in terms of the destination image and its
     highest resolution.
+  cutout: If True, set the bounds of the destination volume to the
+    specified bounds when creating a new volume (existing is untouched).
   translate: Vec3 pointing from source bounds to dest bounds
     and is in terms of the highest resolution of the source image.
     This allows you to compensate for differing voxel offsets
@@ -753,6 +770,8 @@ def create_transfer_tasks(
 
   if dest_voxel_offset:
     dest_voxel_offset = Vec(*dest_voxel_offset, dtype=int)
+  elif cutout and bounds:
+    dest_voxel_offset = list(bounds.minpt)
   else:
     dest_voxel_offset = src_vol.voxel_offset.clone()
 
@@ -771,6 +790,9 @@ def create_transfer_tasks(
   except cloudvolume.exceptions.InfoUnavailableError:
     info = copy.deepcopy(src_vol.info)
     dest_vol = CloudVolume(dest_layer_path, info=info, mip=mip)
+    if cutout and bounds:
+      dest_vol.scale["voxel_offset"] = list(bounds.minpt)
+      dest_vol.scale["size"] = list(bounds.size3())
     dest_vol.commit_info()
 
   if dest_voxel_offset is not None:
@@ -1442,3 +1464,80 @@ def create_ccl_relabel_tasks(
 
   return RelabelCCLTaskIterator(bounds, shape)
 
+def create_voxel_counting_tasks(
+  cloudpath, mip
+):
+  """
+  Counts voxels in 512^3 tasks and uploads the JSON files
+  to $KEY/stats/voxel_counts/$BBOX.json
+  """
+  vol = CloudVolume(cloudpath, max_redirects=0, mip=mip)
+  shape = Vec(512,512,512)
+  bounds = vol.bounds.clone()
+
+  class CountVoxelsTaskIterator(FinelyDividedTaskIterator):
+    def task(self, shape, offset):
+      bounded_shape = min2(shape, bounds.maxpt - offset)
+      return partial(CountVoxelsTask,
+        cloudpath=cloudpath,
+        shape=bounded_shape.clone(),
+        offset=offset.clone(),
+        mip=mip,
+      )
+
+    def on_finish(self):
+      vol = CloudVolume(cloudpath, max_redirects=0)
+      vol.provenance.processing.append({
+        'method': {
+          'task': 'CountVoxelsTask',
+          'cloudpath': cloudpath,
+          'mip': mip,
+          'shape': shape.tolist(),
+        },
+        'by': operator_contact(),
+        'date': strftime('%Y-%m-%d %H:%M %Z'),
+      })
+      vol.commit_provenance()
+
+  return CountVoxelsTaskIterator(bounds, shape)
+
+def accumulate_voxel_counts(cloudpath, mip, progress=True):
+  """
+  Accumulate counts from each task.
+
+  Results are saved in a mapbuffer file:
+  $cloudpath/$KEY/stats/voxel_counts.mb
+  """
+  vol = CloudVolume(cloudpath, max_redirects=0, mip=mip)
+  shape = Vec(512,512,512)
+  bounds = vol.bounds.clone()
+
+  class CountVoxelsTaskIterator(FinelyDividedTaskIterator):
+    def task(self, shape, offset):
+      bounded_shape = min2(shape, bounds.maxpt - offset)
+      return Bbox(offset, offset + bounded_shape)
+  
+  itr = CountVoxelsTaskIterator(bounds, shape)
+  filenames = ( f'{bbx.to_filename()}.json' for bbx in itr )
+
+  cf = CloudFiles(cloudpath)
+  stats_path = cf.join(cloudpath, f'{vol.key}', 'stats', 'voxel_counts')
+  cf = CloudFiles(stats_path, progress=progress)
+
+  count_files = cf.get_json(filenames, total=len(itr))
+  final_counts = defaultdict(int)
+
+  for counts in tqdm(count_files, disable=(not progress), desc="Summing"):
+    for segid, ct in counts.items():
+      final_counts[int(segid)] += ct
+
+  final_path = cf.join(cloudpath, f'{vol.key}', 'stats')
+  cf = CloudFiles(final_path)
+
+  mb = MapBuffer(
+    final_counts, 
+    tobytesfn=lambda x: x.to_bytes(8, byteorder='little'),
+    check_crc=False,
+  )
+  cf.put('voxel_counts.mb', mb.tobytes())
+
