@@ -138,12 +138,6 @@ def process_mesh(
         frag=frag_no,
       )
 
-      # test for totally degenerate meshes by checking if 
-      # all of two axes match, meaning the mesh must be a 
-      # point or a line.
-      if np.sum(np.bitwise_and.reduce(submesh.vertices) == submesh.vertices[0,:]) >= 2:
-        continue
-
       minpt = np.min(submesh.vertices, axis=0)
       quantization_range = np.max(submesh.vertices, axis=0) - minpt
       quantization_range = np.max(quantization_range)
@@ -160,7 +154,7 @@ def process_mesh(
           create_metadata=True,
         )
       except DracoPy.EncodingFailedException:
-        continue
+        submesh = b''
 
       manifest.fragment_offsets.append(len(submesh))
       mesh_binaries.append(submesh)
@@ -297,7 +291,7 @@ def generate_lods(
   mesh:Mesh,
   num_lods:int,
   decimation_factor:int = 2, 
-  aggressiveness:float = 5.0,
+  aggressiveness:float = 5.5,
   progress:bool = False,
 ):
   assert num_lods >= 0, num_lods
@@ -348,12 +342,12 @@ def create_mesh_shard(
   data_offset = {
     label: len(manifest)
     for label, (manifest, mesh_binary) in meshes.items()
-    if manifest is not None
+    if manifest is not None and len(mesh_binary) > 0
   }
   meshes = {
     label: mesh_binary + manifest.to_binary()
     for label, (manifest, mesh_binary) in meshes.items()
-    if manifest is not None
+    if manifest is not None and len(mesh_binary) > 0
   }
 
   if len(meshes) == 0:
@@ -522,9 +516,17 @@ def create_octree_level_from_mesh(mesh, chunk_shape, lod, num_lods):
         mesh_z = trimesh.intersections.slice_mesh_plane(mesh_y, plane_normal=nz, plane_origin=list(nz*z*scale.z+oz))
         mesh_z = trimesh.intersections.slice_mesh_plane(mesh_z, plane_normal=-nz, plane_origin=list(nz*(z+1)*scale.z+oz))
 
-        if len(mesh_z.vertices) > 0:
-          submeshes.append(mesh_z)
-          nodes.append((x, y, z))
+        if len(mesh_z.vertices) == 0:
+          continue
+
+        # test for totally degenerate meshes by checking if 
+        # all of two axes match, meaning the mesh must be a 
+        # point or a line.
+        if np.sum([ np.all(mesh_z.vertices[:,i] == mesh_z.vertices[0,i]) for i in range(3) ]) >= 2:
+          continue
+
+        submeshes.append(mesh_z)
+        nodes.append((x, y, z))
 
   # Sort in Z-curve order
   submeshes, nodes = zip(
