@@ -62,9 +62,27 @@ class CloudPath(click.ParamType):
   def convert(self, value, param, ctx):
     return cloudfiles.paths.normalize(value)
 
+def compute_bounds(path, mip, xrange, yrange, zrange):
+  bounds = None
+  if xrange or yrange or zrange:
+    bounds = CloudVolume(path).meta.bounds(mip)
+
+  if xrange:
+    bounds.minpt.x = xrange[0]
+    bounds.maxpt.x = xrange[1]
+  if yrange:
+    bounds.minpt.y = yrange[0]
+    bounds.maxpt.y = yrange[1]
+  if zrange:
+    bounds.minpt.z = zrange[0]
+    bounds.maxpt.z = zrange[1]
+
+  return bounds
+
+
 @click.group()
 @click.option("-p", "--parallel", default=1, help="Run with this number of parallel processes. If 0, use number of cores.")
-@click.version_option(version="4.11.1")
+@click.version_option(version="4.13.0")
 @click.pass_context
 def main(ctx, parallel):
   """
@@ -164,19 +182,7 @@ def downsample(
   if volumetric:
   	factor = (2,2,2)
 
-  bounds = None
-  if xrange or yrange or zrange:
-    bounds = CloudVolume(path).meta.bounds(mip)
-
-  if xrange:
-    bounds.minpt.x = xrange[0]
-    bounds.maxpt.x = xrange[1]
-  if yrange:
-    bounds.minpt.y = yrange[0]
-    bounds.maxpt.y = yrange[1]
-  if zrange:
-    bounds.minpt.z = zrange[0]
-    bounds.maxpt.z = zrange[1]
+  bounds = compute_bounds(path, mip, xrange, yrange, zrange)
 
   if sharded:
     tasks = tc.create_image_shard_downsample_tasks(
@@ -225,6 +231,11 @@ def downsample(
 @click.option('--clean-info', is_flag=True, default=False, help="Scrub info file of mesh and skeleton fields.", show_default=True)
 @click.option('--no-src-update', is_flag=True, default=False, help="Don't update the source provenance file with the transfer metadata.", show_default=True)
 @click.option('--truncate-scales/--no-truncate-scales', is_flag=True, default=True, help="Truncate the info file scales to the mip specified.", show_default=True)
+@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
+@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
+@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
+@click.option('--bounds-mip', default=None, type=int, help="Which mip level are xrange, yrange, and zrange specified in?", show_default=True)
+@click.option('--cutout', is_flag=True, default=False, help="If bounds are specified and creating a new volume, restrict the new volume to the specified bounds. Currently only for unsharded transfers.", show_default=True)
 @click.pass_context
 def xfer(
 	ctx, src, dest, queue, translate, 
@@ -233,7 +244,8 @@ def xfer(
   encoding, encoding_level, chunk_size, compress,
   volumetric, delete_bg, bg_color, sharded,
   dest_voxel_offset, clean_info, no_src_update,
-  truncate_scales
+  truncate_scales, 
+  xrange, yrange, zrange, bounds_mip, cutout
 ):
   """
   Copy, re-encode, or shard an image layer.
@@ -263,6 +275,10 @@ def xfer(
   if compress and compress.lower() in ("none", "false"):
     compress = False
 
+  if bounds_mip is None:
+    bounds_mip = mip
+  bounds = compute_bounds(src, bounds_mip, xrange, yrange, zrange)
+
   if sharded:
     tasks = tc.create_image_shard_transfer_tasks(
       src, dest,
@@ -270,6 +286,8 @@ def xfer(
       dest_voxel_offset=dest_voxel_offset, translate=translate, 
       encoding=encoding, memory_target=memory, clean_info=clean_info,
       encoding_level=encoding_level, truncate_scales=truncate_scales,
+      compress=compress, bounds=bounds, bounds_mip=bounds_mip,
+      cutout=cutout,
     )
   else:
     tasks = tc.create_transfer_tasks(
@@ -282,12 +300,47 @@ def xfer(
       memory_target=memory, max_mips=max_mips, 
       clean_info=clean_info, no_src_update=no_src_update,
       encoding_level=encoding_level, truncate_scales=truncate_scales,
+      bounds=bounds, bounds_mip=bounds_mip, cutout=cutout,
     )
 
   parallel = int(ctx.obj.get("parallel", 1))
   tq = TaskQueue(normalize_path(queue))
   tq.insert(tasks, parallel=parallel)
 
+@imagegroup.group("voxels")
+def voxelgroup():
+  """Compute voxel counts per label."""
+  pass
+
+@voxelgroup.command("count")
+@click.argument("path", type=CloudPath())
+@click.option('--mip', default=0, help="Count this mip level of the image pyramid.", show_default=True)
+@click.option('--queue', default=None, required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue")
+@click.pass_context
+def count_voxels(ctx, path, mip, queue):
+  """Create voxel counting tasks.
+
+  These tasks are 512x512x512 voxels and result
+  in a JSON file that lives at:
+  $cloudpath/$KEY/stats/voxel_counts/$BBOX.json
+  """
+  tasks = tc.create_voxel_counting_tasks(path, mip=mip)
+  parallel = int(ctx.obj.get("parallel", 1))
+  tq = TaskQueue(normalize_path(queue))
+  tq.insert(tasks, parallel=parallel)
+
+@voxelgroup.command("sum")
+@click.argument("path", type=CloudPath())
+@click.option('--mip', default=0, help="Count this mip level of the image pyramid.", show_default=True)
+@click.pass_context
+def sum_voxel_counts(ctx, path, mip):
+  """Accumulate counts from each task.
+
+  Results are saved in a mapbuffer file:
+  $cloudpath/$KEY/stats/voxel_counts.mb
+  """
+  tc.accumulate_voxel_counts(path, mip=mip)
+
 @imagegroup.group("contrast")
 def contrastgroup():
   """Perform contrast correction on the image."""
@@ -298,29 +351,19 @@ def contrastgroup():
 @click.option('--queue', default=None, required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue")
 @click.option('--mip', default=0, help="Build histogram from this level of the image pyramid.", show_default=True)
 @click.option('--coverage', default=0.01, type=float, help="Fraction of the image to sample. Range: [0,1]", show_default=True)
-@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
-@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
-@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
-@click.option('--bounds-mip', default=0, help="Build upward from this level of the image pyramid.", show_default=True)
+@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
+@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
+@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
+@click.option('--bounds-mip', default=None, type=int, help="Bounds are written in terms of this image pyramid level.", show_default=True)
 @click.pass_context
 def histogram(
   ctx, path, queue, mip, coverage,
   xrange, yrange, zrange, bounds_mip
 ):
   """(1) Compute the histogram for each z-slice."""
-  bounds = None
-  if xrange or yrange or zrange:
-    bounds = CloudVolume(path).meta.bounds(mip)
-
-  if xrange:
-    bounds.minpt.x = xrange[0]
-    bounds.maxpt.x = xrange[1]
-  if yrange:
-    bounds.minpt.y = yrange[0]
-    bounds.maxpt.y = yrange[1]
-  if zrange:
-    bounds.minpt.z = zrange[0]
-    bounds.maxpt.z = zrange[1]
+  if bounds_mip is None:
+    bounds_mip = mip
+  bounds = compute_bounds(path, bounds_mip, xrange, yrange, zrange)
 
   tasks = tc.create_luminance_levels_tasks(
     path, 
@@ -346,10 +389,10 @@ def histogram(
 @click.option('--fill-missing', is_flag=True, default=False, help="Interpret missing image files as background instead of failing.")
 @click.option('--minval', default=None, help="Set left side of histogram as this value. (must be in range of datatype)", show_default=True)
 @click.option('--maxval', default=None, help="Set right side of histogram as this value. (must be in range of datatype)", show_default=True)
-@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
-@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
-@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
-@click.option('--bounds-mip', default=0, help="Build upward from this level of the image pyramid.", show_default=True)
+@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
+@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
+@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected bounds mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
+@click.option('--bounds-mip', default=None, type=int, help="Bounds are written in terms of this image pyramid level.", show_default=True)
 @click.pass_context
 def equalize(
   ctx, src, dest, queue,
@@ -359,19 +402,9 @@ def equalize(
   xrange, yrange, zrange, bounds_mip
 ):
   """(2) Apply histogram equalization to z-slices."""
-  bounds = None
-  if xrange or yrange or zrange:
-    bounds = CloudVolume(path).meta.bounds(mip)
-
-  if xrange:
-    bounds.minpt.x = xrange[0]
-    bounds.maxpt.x = xrange[1]
-  if yrange:
-    bounds.minpt.y = yrange[0]
-    bounds.maxpt.y = yrange[1]
-  if zrange:
-    bounds.minpt.z = zrange[0]
-    bounds.maxpt.z = zrange[1]
+  if bounds_mip is None:
+    bounds_mip = mip
+  bounds = compute_bounds(path, bounds_mip, xrange, yrange, zrange)
 
   tasks = tc.create_contrast_normalization_tasks(
     src, dest, levels_path=None,
@@ -750,6 +783,7 @@ def mesh_xfer(
 @click.option('--fill-missing', is_flag=True, default=False, help="Interpret missing image files as background instead of failing.")
 @click.option('--max-error', default=40, help="Maximum simplification error in physical units.", show_default=True)
 @click.option('--dust-threshold', default=None, help="Skip meshing objects smaller than this number of voxels within a cutout. No default limit. Typical value: 1000.", type=int)
+@click.option('--dust-global/--dust-local', is_flag=True, default=False, help="Use global voxel counts for the dust threshold (when >0). To use this feature you must first compute the global voxel counts using the 'igneous image voxels' command.", show_default=True)
 @click.option('--dir', default=None, help="Write meshes into this directory instead of the one indicated in the info file.")
 @click.option('--compress', default="gzip", help="Set the image compression scheme. Options: 'none', 'gzip', 'br'", show_default=True)
 @click.option('--spatial-index/--skip-spatial-index', is_flag=True, default=True, help="Create the spatial index.", show_default=True)
@@ -760,7 +794,8 @@ def mesh_forge(
   ctx, path, queue, mip, shape, 
   simplify, fill_missing, max_error, 
   dust_threshold, dir, compress, 
-  spatial_index, sharded, closed_edge
+  spatial_index, sharded, closed_edge,
+  dust_global
 ):
   """
   (1) Synthesize meshes from segmentation cutouts.
@@ -774,8 +809,6 @@ def mesh_forge(
   memory efficient as it can use a 32-bit mesher.
 
   zmesh is used: https://github.com/seung-lab/zmesh
-
-  Sharded format not currently supports. Coming soon.
   """
   if compress.lower() == "none":
     compress = False
@@ -786,7 +819,8 @@ def mesh_forge(
     mesh_dir=dir, cdn_cache=False, dust_threshold=dust_threshold,
     object_ids=None, progress=False, fill_missing=fill_missing,
     encoding='precomputed', spatial_index=spatial_index, 
-    sharded=sharded, compress=compress, closed_dataset_edges=closed_edge
+    sharded=sharded, compress=compress, closed_dataset_edges=closed_edge,
+    dust_global=dust_global
   )
 
   parallel = int(ctx.obj.get("parallel", 1))
@@ -976,6 +1010,7 @@ def skeletongroup():
 @click.option('--fix-avocados', is_flag=True, default=False, help="Fixes somata where nuclei and cytoplasm have separate segmentations.", show_default=True)
 @click.option('--fill-holes', is_flag=True, default=False, help="Preprocess each cutout to eliminate background holes and holes caused by entirely contained inclusions. Warning: May remove labels that are considered inclusions.", show_default=True)
 @click.option('--dust-threshold', default=1000, help="Skip skeletonizing objects smaller than this number of voxels within a cutout.", type=int, show_default=True)
+@click.option('--dust-global/--dust-local', is_flag=True, default=False, help="Use global voxel counts for the dust threshold (when >0). To use this feature you must first compute the global voxel counts using the 'igneous image voxels' command.", show_default=True)
 @click.option('--spatial-index/--skip-spatial-index', is_flag=True, default=True, help="Create the spatial index.", show_default=True)
 @click.option('--scale', default=4, help="Multiplies invalidation radius by distance from boundary.", type=float, show_default=True)
 @click.option('--const', default=10, help="Adds constant amount to invalidation radius in physical units.", type=float, show_default=True)
@@ -988,7 +1023,7 @@ def skeletongroup():
 @click.pass_context
 def skeleton_forge(
   ctx, path, queue, mip, shape, 
-  fill_missing, dust_threshold, spatial_index,
+  fill_missing, dust_threshold, dust_global, spatial_index,
   fix_branching, fix_borders, fix_avocados, 
   fill_holes, scale, const, soma_detect, soma_accept,
   soma_scale, soma_const, max_paths, sharded
@@ -1033,6 +1068,7 @@ def skeleton_forge(
     dust_threshold=dust_threshold, progress=False,
     parallel=1, fill_missing=fill_missing, 
     sharded=sharded, spatial_index=spatial_index,
+    dust_global=dust_global,
   )
 
   parallel = int(ctx.obj.get("parallel", 1))
